// Home Control 98
// By Steven Lawrance

// Header Files
#include <windows.h>
#include "x10dll.h"
#include "control.h"

// Mode Types
#define MODE_TIMEREVENT     0
#define MODE_DIRECTCOMMAND  1
#define MODE_READEVERYTHING 2
#define MODE_RESETUNIT      3

// Info Data Types (includes 32-character name + 1 terminating NULL)
typedef struct _CP290UNITINFO
{
  char Name[33];
  char HouseCode;
  char UnitCode;
} CP290UNITINFO;

typedef struct _CP290TIMEREVENTINFO
{
  char Name[33];
  CP290TIMEREVENT Info;
} CP290TIMEREVENTINFO;

// Thread Prototype
int InteractingThread(int Action);

// Dialog Box Prototypes
BOOL CALLBACK MainDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CommandDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK PropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK UnitsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK UnitPropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TimersDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TimerPropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK InteractingDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK DaysDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ResetAskDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
//BOOL CALLBACK IconEditDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

// Miscellaneous Prototypes
void LoadPortName(LPSTR szCommPort);
void SavePortName(LPSTR szCommPort);
void LoadGlobals(void);
void SaveGlobals(void);
void ClearGlobals(HWND hWnd);
void DisplayError(HWND hDlg, LPCSTR Title);
BOOL Interact(HWND hWnd, DWORD Action);
void StringToIconArray(LPCSTR str, LPCP290ICON icons, BOOL hex);
void IconArrayToString(LPCP290ICON icons, LPSTR str, BOOL hex);

// Global Variables
CP290UNITINFO gUnit[16][16];
BOOL gSelected[16][16];
CP290TIMEREVENTINFO gEvt[128];
CP290TIMEREVENT gConfigEvt;
//CP290ICON gIcon[256];
HINSTANCE ghInstance;
CP290COMMAND gCmd;
HANDLE ghThread;
HWND ghInteractingDlg;
BOOL RetainMemory;
BOOL RetainIcons;
BOOL RetainEvents;
HANDLE ghAnimationThread;
char NewHouseCode;
int x;
DWORD ThreadID, AnimationThreadID;
BOOL ErrorFlag;

// Program Entry Point

int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
  // Initialize the custom control library and set the ghInstance variable
  ghInstance = hInstance;
  ghAnimationThread = NULL;
  ThreadID = 0;
  AnimationThreadID = 0;
  ErrorFlag = FALSE;
  
  // Initialize the global variables
  ClearGlobals(HWND_DESKTOP);

  // Run the main dialog box
  DialogBox(hInstance, "MAIN", HWND_DESKTOP, (DLGPROC) MainDlgProc);
  
  // Save the global variables
  SaveGlobals();

  // Close animation thread
  if (ghAnimationThread)
	CloseHandle(ghAnimationThread);
  
  // Exit
  return 0;
}

void ClearGlobals(HWND hWnd)
{
  int lcv[2];
  
  // Clear and initialize the global arrays
  for (lcv[0] = 0; lcv[0] <= 15; lcv[0]++)
  {
    for (lcv[1] = 0; lcv[1] <= 15; lcv[1]++)
    {
      ZeroMemory(&gUnit[lcv[0]][lcv[1]], sizeof (CP290UNITINFO));
      gUnit[lcv[0]][lcv[1]].HouseCode = (char) lcv[0];
      gUnit[lcv[0]][lcv[1]].UnitCode = (char) lcv[1];
      gSelected[lcv[0]][lcv[1]] = FALSE;
    }
  }
  for (lcv[0] = 0; lcv[0] <= 127; lcv[0]++)
  {
    ZeroMemory(&gEvt[lcv[0]], sizeof(CP290TIMEREVENTINFO));
    gEvt[lcv[0]].Info.EventId = (char) lcv[0];
    gEvt[lcv[0]].Info.HouseCode = 'A';
  }
  ZeroMemory(&gCmd, sizeof(CP290COMMAND));
  
  // Try to load everything from the unit; if there is a problem, ask the user for a retry
  do
  {
	ErrorFlag = 2;
	if (!Interact(ghInteractingDlg, MODE_READEVERYTHING))
	{
	  if (MessageBox(hWnd, "Do you want to change the port that the CP290 is connected to?", "Solution to Problem?", MB_YESNO | MB_ICONQUESTION) == IDYES)
	  {
		ErrorFlag = 2;
		DialogBox(ghInstance, "CONFIG", hWnd, (DLGPROC) PropertiesDlgProc);
	  } else
		ErrorFlag = TRUE;
	} else
	  ErrorFlag = FALSE;
  } while (ErrorFlag == 2);
  
  // Load the registry's values
  LoadGlobals();
}

void DisplayError(HWND hDlg, LPCSTR Title)
{
  if (ThreadID) {
	char ErrorMessage[CP290ERR_MESSAGELENGTH];
	CP290_GetLastError(ErrorMessage);
	MessageBox(hDlg, ErrorMessage, Title, MB_OK | MB_ICONSTOP);
	SetDlgItemText(hDlg, IDC_ACTION, "Exiting");
	SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);

	// Uninitialize the comm port
    CP290_Uninitialize();

	// Set the error flag
	ErrorFlag = TRUE;

    // Set the ThreadID to NULL and end the thread
    ThreadID = 0;
	ExitThread(FALSE);
  }
}


int InteractingThread(int Action)
{
  int lcv;
  char Port[7];
  
  // Set the thread's priority to a higher value (preemptive multitasking operating
  // systems allow programs to request higher priorities)
  SetThreadPriority(ghThread, THREAD_PRIORITY_HIGHEST);
  ErrorFlag = FALSE;

  // Initialize the unit
  SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Initializing");
  LoadPortName(Port);
  if (!CP290_Initialize(Port))
	DisplayError(ghInteractingDlg, "Problem During Initiailization");

  // Perform whatever action the calling procedure wanted us to do...
  switch (Action)
  {
  case MODE_TIMEREVENT :
    {
      // Write the event to the unit
      SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Event");
      do
      {
        ErrorFlag = FALSE;
        if (!CP290_SetTimerEvent(&gEvt[x].Info))
          ErrorFlag = TRUE;
        
        // If the event could not be set, then ask the user if they want to
        // try again
        if (ErrorFlag && ThreadID)
        {
          char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
          CP290_GetLastError(ErrorMessage);
          lstrcat(ErrorMessage, ".  Your event could not be written properly.  Do you want to try again?");
          if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Event Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
            ErrorFlag = FALSE;
        }
      } while (ErrorFlag && ThreadID);
      break;
    }
  case MODE_DIRECTCOMMAND :
    {
      // Perform a direct command
      SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Performing Command");
      do
      {
        ErrorFlag = FALSE;
        if (!CP290_Control(&gCmd))
          ErrorFlag = TRUE;
        
        // If the command encountered error, then ask the user if they want to
        // try again
        if (ErrorFlag && ThreadID)
        {
          char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
          CP290_GetLastError(ErrorMessage);
          lstrcat(ErrorMessage, ".  Your command may have not executed properly.  Do you want to try again?");
          if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Event Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
            ErrorFlag = FALSE;
        }
      } while (ErrorFlag && ThreadID);
      break;
    }
  case MODE_READEVERYTHING :
    {
      // Read the events
      SYSTEMTIME cpTime;
      CP290TIMEREVENT cpEvent[128];
//	  CP290ICON cpIcon[256];

      // Fetch base housecode
      SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Fetching Base Housecode");
      if (!(gCmd.HouseCode = CP290_GetBaseHouseCode())) {
        DisplayError(ghInteractingDlg, "Problem During Base Housecode Fetch");
		gCmd.HouseCode = 'A';
	  }

      // Read the icons
/*	  if (ThreadID) {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading Icons");
        if (!CP290_GetIcons(cpIcon, 256))
          DisplayError(ghInteractingDlg, "Problem During Icon Read");
	  }

      // Copy the icons into the global variables
      for (lcv = 0; (lcv <= 255) && ThreadID; lcv++)
        CopyMemory(&gIcon[lcv], &cpIcon[lcv], sizeof(CP290ICON));*/

      // Read the events
	  if (ThreadID) {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading Events");
        if (!CP290_GetTimerEvents(cpEvent, 128))
          DisplayError(ghInteractingDlg, "Problem During Event Read");
	  }
      
      // Copy the timer events into the global variables
      for (lcv = 0; (lcv <= 127) && ThreadID; lcv++)
        CopyMemory(&gEvt[lcv].Info, &cpEvent[lcv], sizeof(CP290TIMEREVENT));
      
      // Set the time to the time of the current time zone
	  if (ThreadID) {
        GetLocalTime(&cpTime);
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Time");
	  }
      do
      {
		if (ThreadID)
          ErrorFlag = !CP290_SetTime(&cpTime);
        
        // If the time could not be set, then ask the user if they want to
        // try again
        if (ErrorFlag && ThreadID)
        {
          char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
          CP290_GetLastError(ErrorMessage);
          lstrcat(ErrorMessage, ".  The time might not have been set properly.  Do you want to try again?");
          if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Time Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
            ErrorFlag = FALSE;
        }
      } while (ErrorFlag && ThreadID);
      break;
    }
  case MODE_RESETUNIT :
    {
      // Reset the unit
      CP290TIMEREVENT cpEvent[128];
      CP290ICON cpIcon[256];
      SYSTEMTIME cpTime;
      char HouseCode;
      
      if (RetainMemory && RetainEvents && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading Events");
        if (!CP290_GetTimerEvents(cpEvent, 128))
          DisplayError(ghInteractingDlg, "Problem During Event Read");
      }
      if (RetainMemory && RetainIcons && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading Icons");
        if (!CP290_GetIcons(cpIcon, 256))
          DisplayError(ghInteractingDlg, "Problem During Icon Read");
      }
      if (RetainMemory && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading Time");
        if (!CP290_GetTime(&cpTime))
          DisplayError(ghInteractingDlg, "Problem During Time Read");
		if (ghInteractingDlg) {
          SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Reading House Code");
          if (!(HouseCode = CP290_GetBaseHouseCode()))
            DisplayError(ghInteractingDlg, "Problem During House Code Read");
		}
      }
      
	  if (ThreadID) {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Resetting");
		
		if (!CP290_Reset())
		{
		  if ((CP290_GetLastError(NULL) == CP290ERR_WRITE) && ThreadID)
		  {
			MessageBox(ghInteractingDlg, "There was a problem during the reset.  However, your data is completely safe.  Please check the connection to the unit and try again.", "Problem During Reset", MB_OK | MB_ICONSTOP);
			SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Exiting");
			SendMessage(ghInteractingDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
			ExitThread(0);
		  }
		  else
		  {
			MessageBox(ghInteractingDlg, "There was an undocumented problem during the reset, which means that you are using a newer version of X10DLL.DLL.  Your data might have been erased.  Please check the connection to the unit and try again.  If you chose to retain the settings, then I will restore them.", "Problem During Reset", MB_OK | MB_ICONSTOP);
			if (!RetainMemory && ThreadID)
			{
			  SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Exiting");
			  SendMessage(ghInteractingDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
			  ExitThread(0);
			}
		  }
		}
	  }
      
      if (RetainMemory && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing House Code");
        if (NewHouseCode != 'A' - 1)
          HouseCode = NewHouseCode;
        do
        {
          ErrorFlag = !CP290_SetBaseHouseCode(HouseCode);
          if (ErrorFlag && ThreadID)
          {
            char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
            CP290_GetLastError(ErrorMessage);
            lstrcat(ErrorMessage, ".  The house code might not have been set properly.  Do you want to try again?");
            if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During House Code Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
              ErrorFlag = FALSE;
          }
        } while (ErrorFlag && ThreadID);
        
        if (ThreadID) {
		  SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Time");
		  do
		  {
			ErrorFlag = !CP290_SetTime(&cpTime);
			if (ErrorFlag && ThreadID)
			{
			  char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
			  CP290_GetLastError(ErrorMessage);
			  lstrcat(ErrorMessage, ".  The time might not have been set properly.  Do you want to try again?");
			  if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Time Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
				ErrorFlag = FALSE;
			}
		  } while (ErrorFlag && ThreadID);
		}
      }
      
      if (RetainMemory && RetainEvents && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Events");
        do
        {
          ErrorFlag = FALSE;
          for (lcv = 0; (lcv <= 127) && ThreadID; lcv++) {
            if (cpEvent[lcv].InUse)
              if (!CP290_SetTimerEvent(&cpEvent[lcv]))
                ErrorFlag = TRUE;
		  }
		  if (ErrorFlag && ThreadID)
		  {
			char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
			CP290_GetLastError(ErrorMessage);
			lstrcat(ErrorMessage, ".  One or more of your events could not be written properly.  Do you want to try again?");
			if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Event Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			  ErrorFlag = FALSE;
		  }
        } while (ErrorFlag && ThreadID);
      }
      
      if (RetainMemory && RetainIcons && ThreadID)
      {
        SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Icons");
        do
        {
          ErrorFlag = FALSE;
          for (lcv = 0; (lcv <= 255) && ThreadID; lcv++) {
            if (cpIcon[lcv].InUse)
              if (!CP290_SetIcon(&cpIcon[lcv]))
                ErrorFlag = TRUE;
		  }
		  if (ErrorFlag && ThreadID)
		  {
			char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
			CP290_GetLastError(ErrorMessage);
			lstrcat(ErrorMessage, ".  One or more of your icons could not be written properly.  Do you want to try again?");
			if (MessageBox(ghInteractingDlg, ErrorMessage, "Problem During Icon Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			  ErrorFlag = FALSE;
		  }
        } while (ErrorFlag && ThreadID);
      }
      
      if (!RetainMemory && ThreadID)
      {
        if (MessageBox(ghInteractingDlg, "Would you like to initialize the unit's values to their defaults?", "Initialization", MB_YESNO | MB_ICONQUESTION) == IDYES)
        {
          SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing House Code");
          if (NewHouseCode != 'A' - 1)
            CP290_SetBaseHouseCode(NewHouseCode);
          else
            CP290_SetBaseHouseCode('A');
          SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Writing Time");
          GetLocalTime(&cpTime);
          CP290_SetTime(&cpTime);
        }
      }
    }
  }
  
  // Exit
  SetDlgItemText(ghInteractingDlg, IDC_ACTION, "Finishing");
  SendMessage(ghInteractingDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);

  // Uninitialize the comm port
  CP290_Uninitialize();

  // If the ThreadID is set, then return TRUE; otherwise, return FALSE
  if (ThreadID) {
    // Set the ThreadID to NULL
    ThreadID = 0;
    return TRUE;
  } else
	return FALSE;
}


BOOL CALLBACK MainDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDC_COMMAND :
        {
          // Send a command
          DialogBox(ghInstance, "COMMAND", hDlg, (DLGPROC) CommandDlgProc);
          break;
        }
      case IDC_TIMER :
        {
          // Brings up the timers dialog box
          DialogBox(ghInstance, "TIMERS", hDlg, (DLGPROC) TimersDlgProc);
          break;
        }
      case IDC_PROPERTIES :
        {
          // Bring up the properties
          DialogBox(ghInstance, "CONFIG", hDlg, (DLGPROC) PropertiesDlgProc);
          break;
        }
/*	  case IDC_ICONS :
		{
		  // Edit the icons
          DialogBox(ghInstance, "ICONEDITOR", hDlg, (DLGPROC) IconEditDlgProc);
          break;
		}*/
      case IDCANCEL :
        {
          // Exit the program
          EndDialog(hDlg, 0);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}

void IconArrayToString(LPCP290ICON icons, LPSTR str, BOOL hex) {
  // Convert an icon array into a string
  DWORD i, lineThrow = 0;
  for (i = 0; i <= 255; i++) {
	if (icons[i].InUse) {
	  if (!hex) {
		if (icons[i].Data[0] < 32)
		  str[i * 2 + lineThrow] = '.';
		else
	      str[i * 2 + lineThrow] = icons[i].Data[0];
		if (icons[i].Data[1] < 32)
		  str[i * 2 + 1 + lineThrow] = '.';
		else
	      str[i * 2 + 1 + lineThrow] = icons[i].Data[1];
		if ((i + 1) % 10 == 0) {
		  str[i * 2 + 1 + lineThrow] = '\r';
		  str[i * 2 + 2 + lineThrow] = '\n';
		  lineThrow += 2;
		}
	  } else
		wsprintf((LPSTR) (str + i * 6), "%02X %02X%c%c", icons[i].Data[0], icons[i].Data[1], ((i + 1) % 10 == 0)? '\r': (i == 255)? 0: ' ', ((i + 1) % 10 == 0)? '\n': 0);
	} else {
	  if (!hex) {
	    str[i * 2 + lineThrow] = '.';
	    str[i * 2 + 1 + lineThrow] = '.';
		if ((i + 1) % 5 == 0) {
		  str[i * 2 + 1 + lineThrow] = '\r';
		  str[i * 2 + 2 + lineThrow] = '\n';
		  lineThrow += 2;
		}
	  } else
		wsprintf((LPSTR) (str + i * 6), "00 00%c%c", (i == 255)? 0: (((i + 1) % 5 == 0)? '\r': ' '), ((i + 1) % 5 == 0)? '\n': 0);
	}
  }
  if (!hex)
    str[512] = 0;
}

void StringToIconArray(LPCSTR str, LPCP290ICON icons, BOOL hex) {
  // Convert a string into an icon array
  DWORD i;
  for (i = 0; i <= 255; i++) {
	icons[i].IconId = (char) i;
	if (str[i * 2] != 0) {
	  icons[i].InUse = TRUE;
	  icons[i].Data[0] = str[i * 2];
	  icons[i].Data[1] = str[i * 2 + 1];
	} else {
	  icons[i].InUse = FALSE;
	  icons[i].Data[0] = 0;
	  icons[i].Data[1] = 0;
	}
  }
}

/*BOOL CALLBACK IconEditDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
	  // Set the dialog item font
	  HFONT hFont = CreateFont(14, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, 0, "Courier New");
	  char str[1589];
	  SendMessage(GetDlgItem(hDlg, IDC_ICONVALUE), WM_SETFONT, (WPARAM) hFont, 0);
	  SendMessage(GetDlgItem(hDlg, IDC_ICONTEXT), WM_SETFONT, (WPARAM) hFont, 0);

	  // Write the hex string to the contents window
	  IconArrayToString(gIcon, str, TRUE);
      SetDlgItemText(hDlg, IDC_ICONVALUE, str);
	  SendDlgItemMessage(hDlg, IDC_ICONVALUE, EM_LIMITTEXT, 1587, 0);

	  // Write the text string to the contents window
	  IconArrayToString(gIcon, str, FALSE);
      SetDlgItemText(hDlg, IDC_ICONTEXT, str);
	  SendDlgItemMessage(hDlg, IDC_ICONTEXT, EM_LIMITTEXT, 563, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDCANCEL :
        {
		  // Delete the font
		  HFONT hFont = (HFONT) SendMessage(GetDlgItem(hDlg, IDC_ICONVALUE), WM_GETFONT, 0, 0);
	      SendMessage(GetDlgItem(hDlg, IDC_ICONVALUE), WM_SETFONT, 0, 0);
	      SendMessage(GetDlgItem(hDlg, IDC_ICONTEXT), WM_SETFONT, 0, 0);
		  DeleteObject(hFont);

          // Exit this screen
          EndDialog(hDlg, 0);
          break;
        }
	  case IDC_ICONVALUE :
		{
		  // Notification from the hex view
		  switch (HIWORD(wParam)) {

		  case EN_SETFOCUS :
			SendDlgItemMessage(hDlg, IDC_ICONVALUE, EM_SETSEL, 0, 2);
			break;
		  }
		}
	  case IDC_ICONTEXT :
		{
		  // Notification from the text view
		  switch (HIWORD(wParam)) {

		  case EN_SETFOCUS :
			SendDlgItemMessage(hDlg, IDC_ICONTEXT, EM_SETSEL, 0, 1);
			break;

		  }
		}
      }
      return 0;
    }
  }
  return FALSE;
}*/


BOOL CALLBACK PropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      // Initialize the values of the dialog box
      char Port[7];
      LoadPortName(Port);
	  SetDlgItemText(hDlg, IDC_PORT, (LPCSTR) (Port + 3));
	  SendDlgItemMessage(hDlg, IDC_PORT, EM_LIMITTEXT, 3, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDOK :
        {
		  // Ensure the value is valid
		  DWORD val = GetDlgItemInt(hDlg, IDC_PORT, NULL, FALSE);
		  if ((val >= 1) && (val <= 256)) {
            // Get the selected port and exit
            char Port[7];
			wsprintf(Port, "COM%lu", val);
            SavePortName(Port);
            EndDialog(hDlg, TRUE);
		  } else {
			MessageBox(hDlg, "Comm port must be an integer between 1 and 256.", "Invalid Comm Port", MB_OK | MB_ICONEXCLAMATION);
			SetFocus(GetDlgItem(hDlg, IDC_PORT));
			SendDlgItemMessage(hDlg, IDC_PORT, EM_SETSEL, 0, -1);
		  }
          break;
        }
      case IDC_RESETUNIT :
        {
		  // Ensure the value is valid
		  DWORD val = GetDlgItemInt(hDlg, IDC_PORT, NULL, FALSE);
		  if ((val >= 1) && (val <= 256)) {
            // Launch the Reset Unit box with the selected comm port
            char Port[7];
			wsprintf(Port, "COM%lu", val);
            SavePortName(Port);
            if (DialogBoxParam(ghInstance, "RESETASK", hDlg, (DLGPROC) ResetAskDlgProc, (LPARAM) Port)) {
              // Do the dirty deed of resetting the unit
              Interact(hDlg, MODE_RESETUNIT);
			}

		    // End the dialog box
            EndDialog(hDlg, FALSE);
		  } else {
			MessageBox(hDlg, "Comm port must be an integer between 1 and 256.", "Invalid Comm Port", MB_OK | MB_ICONEXCLAMATION);
			SetFocus(GetDlgItem(hDlg, IDC_PORT));
			SendDlgItemMessage(hDlg, IDC_PORT, EM_SETSEL, 0, -1);
		  }
          break;
        }
      case IDCANCEL :
        {
          // Exit
          EndDialog(hDlg, FALSE);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}


BOOL CALLBACK CommandDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      int i;
      char String[10];
      
      // Set the current button states according to the previous values
      switch (gCmd.Command)
      {
      case CP290_ON  : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
        CheckDlgButton(hDlg, IDC_ON, TRUE);  break;
      case CP290_OFF : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
        CheckDlgButton(hDlg, IDC_OFF, TRUE);  break;
      case CP290_DIM : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), TRUE);
        CheckDlgButton(hDlg, IDC_DIM, TRUE);  break;
      }
      
      // Place the percents into the dim level combo-box
      for (i = 15; i >= 0; i--)
      {
        LoadString(ghInstance, i, String, 10);
        SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_ADDSTRING, 0, (LPARAM) String);
      }
      
      // Select the previous dim level
      SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_SETCURSEL, 15 - gCmd.DimLevel, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
        // Set the states for the pull-down list box according to the
        // selected command
      case IDC_ON :
      case IDC_OFF :
        {
          EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
          break;
        }
      case IDC_DIM :
        {
          EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), TRUE);
          break;
        }
        
        // Run the ChangeUnits dialog box
      case IDC_CHANGEUNITS :
        {
          DialogBox(ghInstance, "UNITS", hDlg, (DLGPROC) UnitsDlgProc);
          break;
        }
        
        // User pressed "Ok"
      case IDOK :
        {
          BOOL UnitSelected;
          int lcv[2];
          
          // Get Command Type
          if (IsDlgButtonChecked(hDlg, IDC_ON))
            gCmd.Command = CP290_ON;
          if (IsDlgButtonChecked(hDlg, IDC_OFF))
            gCmd.Command = CP290_OFF;
          if (IsDlgButtonChecked(hDlg, IDC_DIM))
            gCmd.Command = CP290_DIM;
          
          // Get Dim Level
          gCmd.DimLevel = 15 - SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_GETCURSEL, 0, 0);
          
          UnitSelected = FALSE;
          for (lcv[0] = 0; lcv[0] <= 15; lcv[0]++)
            for (lcv[1] = 0; lcv[1] <= 15; lcv[1]++)
              if (gSelected[lcv[0]][lcv[1]])
                UnitSelected = TRUE;
              
              if (UnitSelected)
              {
                // Interact with the unit and exit if no error messages made their presence known
                if (Interact(hDlg, MODE_DIRECTCOMMAND))
                {
                  // Exit with TRUE
                  EndDialog(hDlg, TRUE);
                }
              }
              else
              {
                // Tell the user to select a unit
                MessageBox(hDlg, "You must select at least one unit to manipulate.", "Unit Error", MB_OK | MB_ICONEXCLAMATION);
              }
              break;
        }
        
        // User pressed "Cancel"
      case IDCANCEL :
        {
          // Exit with FALSE
          EndDialog(hDlg, FALSE);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}


BOOL CALLBACK UnitsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      int i;
      char String[2];
      
      // Place 'A' through 'P' into the drop-down list box
      ZeroMemory(String, 2);
      for (i = 'A'; i <= 'P'; i++)
      {
        String[0] = (char) i;
        SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) String);
      }
      
      // Copy the gCmd units into the Selected array
      for (i = 0; i <= 15; i++)
        gSelected[gCmd.HouseCode][i] = gCmd.Unit[i];
      
      // Select the current house code in the drop-down list box
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_SETCURSEL, gCmd.HouseCode - 'A', 0);
      
      // Let the code in the WM_COMMAND process the rest of the house code change
      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDC_HOUSECODE :
        {
          // If the selection for the house code had changed, then we'll need to do plenty...
          if (HIWORD(wParam) == CBN_SELCHANGE)
          {
            BOOL AnySelection;
            int lcv;
            int HouseCode;
            int ListBoxCurSel[2];
            
            // Back-up the previous selections
            ListBoxCurSel[0] = SendDlgItemMessage(hDlg, IDC_UNITSFREE, LB_GETCURSEL, 0, 0);
            ListBoxCurSel[1] = SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCURSEL, 0, 0);
            
            // Get the new housecode and clear the list boxes
            HouseCode = SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0);
            SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_RESETCONTENT, 0, 0);
            SendDlgItemMessage(hDlg, IDC_UNITSFREE, LB_RESETCONTENT, 0, 0);
            
            // Set AnySelection to FALSE and begin a loop
            AnySelection = FALSE;
            for (lcv = 0; lcv <= 15; lcv++)
            {
              // If a unit name does not exist, then we'll need to assign it its
              // default value: the house code followed by the number
              if (lstrcmp(gUnit[HouseCode][lcv].Name, "") == 0)
              {
                gUnit[HouseCode][lcv].Name[0] = (char) (HouseCode + 'A');
                if (lcv < 9)
                {
                  gUnit[HouseCode][lcv].Name[1] = (char) (lcv + '1');
                  gUnit[HouseCode][lcv].Name[2] = '\0';
                }
                else
                {
                  gUnit[HouseCode][lcv].Name[1] = '1';
                  gUnit[HouseCode][lcv].Name[2] = (char) (lcv - 10 + '1');
                  gUnit[HouseCode][lcv].Name[3] = '\0';
                }
              }
              
              // If the unit is selected, then we'll need to put it into the selection
              // list box and then assign a variable association of its unit code to it.
              // Also, AnySelection is set to TRUE
              if (gSelected[HouseCode][lcv])
              {
                SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_ADDSTRING, 0, (LPARAM) gUnit[HouseCode][lcv].Name);
                SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_SETITEMDATA, (WPARAM) SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCOUNT, 0, 0) - 1, (LPARAM) lcv);
                AnySelection = TRUE;
              }
              
              // Add the unit name to the units list box
              SendDlgItemMessage(hDlg, IDC_UNITSFREE, LB_ADDSTRING, 0, (LPARAM) gUnit[HouseCode][lcv].Name);
            }
            
            // If any selection exists, then we'll need to do this...
            if (AnySelection)
            {
              // If the backed-up selection is beyond the current range, then set it to zero
              if (((SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCOUNT, 0, 0) - 1) < ListBoxCurSel[1]) ||
                (ListBoxCurSel[1] < 0))
                ListBoxCurSel[1] = 0;
              
              // Select the backed-up position for the selection list box
              SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_SETCURSEL, ListBoxCurSel[1], 0);
            }
            
            // Select the backed-up position for the units list box; if the back-up was
            // below zero, then we'll need to select zero
            if (ListBoxCurSel[0] < 0)
              ListBoxCurSel[0] = 0;
            SendDlgItemMessage(hDlg, IDC_UNITSFREE, LB_SETCURSEL, ListBoxCurSel[0], 0);
          }
          break;
        }
      case IDC_PROPERTIES :
        {
          // Bring up the unit properties dialog box and refresh if "Ok" was pressed
          if (DialogBoxParam(ghInstance, "UNITPROPERTIES", hDlg, (DLGPROC) UnitPropertiesDlgProc, (LPARAM) hDlg))
            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
          break;
        }
      case IDC_UNITSFREE :
        {
          // Filter double clicks into IDC_ADDUNIT
          if (HIWORD(wParam) != LBN_DBLCLK)
            break;
        }
      case IDC_ADDUNIT :
        {
          // Select the unit from the list into the selection
          gSelected[SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0)][SendDlgItemMessage(hDlg, IDC_UNITSFREE, LB_GETCURSEL, 0, 0)] = TRUE;
          SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
          break;
        }
      case IDC_REMOVEUNIT :
        {
          // Unselect the selected unit from the selection if any selection exists
          if (SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCOUNT, 0, 0) != 0)
          {
            gSelected[SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0)][SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETITEMDATA, SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCURSEL, 0, 0), 0)] = FALSE;
            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
          }
          break;
        }
      case IDC_CLEAR :
        {
          // Clear the selection if any selection exists
          if (SendDlgItemMessage(hDlg, IDC_UNITSUSED, LB_GETCOUNT, 0, 0) != 0)
          {
            int HouseCode = SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0);
            int lcv;
            for (lcv = 0; lcv <= 15; lcv++)
              gSelected[HouseCode][lcv] = FALSE;
            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
          }
          break;
        }
      case IDOK :
        {
          int lcv;
          int HouseCode;
          
          // Copy the Selected array into gCmd by copying the house code and the units
          HouseCode = SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0);
          gCmd.HouseCode = (char) (HouseCode + 'A');
          for (lcv = 0; lcv <= 15; lcv++)
            gCmd.Unit[lcv] = gSelected[HouseCode][lcv];
          EndDialog(hDlg, TRUE);
          break;
        }
      case IDCANCEL :
        {
          EndDialog(hDlg, FALSE);
          break;
        }
        }
        return 0;
     }
  }
  return FALSE;
}


BOOL CALLBACK UnitPropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      int i;
      char String[3];
      int UnitCode, HouseCode;
      
      // Get the house and unit codes from the selections in the dialog box identified
      // by the lParam parameter
      HouseCode = SendDlgItemMessage((HWND) lParam, IDC_HOUSECODE, CB_GETCURSEL, 0, 0);
      UnitCode = SendDlgItemMessage((HWND) lParam, IDC_UNITSFREE, LB_GETCURSEL, 0, 0);
      
      // Place 'A' through 'P' into the drop-down list box
      ZeroMemory(String, 2);
      for (i = 'A'; i <= 'P'; i++)
      {
        String[0] = (char) i;
        SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) String);
      }
      
      // Place 1 through 16 into the drop-down list box
      ZeroMemory(String, 3);
      for (i = 1; i <= 16; i++)
      {
        if (i <= 9)
          String[0] = (char) (i + '0');
        else
        {
          String[0] = '1';
          String[1] = (char) (i - 10 + '0');
        }
        SendDlgItemMessage(hDlg, IDC_UNITCODE, CB_ADDSTRING, 0, (LPARAM) String);
      }
      
      // Select the defaults into the drop-down list boxes
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_SETCURSEL, HouseCode, 0);
      SendDlgItemMessage(hDlg, IDC_UNITCODE, CB_SETCURSEL, UnitCode, 0);
      
      // Set the name and limit the text to 32
      SetDlgItemText(hDlg, IDC_UNITNAME, gUnit[HouseCode][UnitCode].Name);
      SendDlgItemMessage(hDlg, IDC_UNITNAME, EM_LIMITTEXT, 32, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDOK :
        {
          int HouseCode, UnitCode;
          
          // Rename the selected unit
          HouseCode = SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0);
          UnitCode = SendDlgItemMessage(hDlg, IDC_UNITCODE, CB_GETCURSEL, 0, 0);
          GetDlgItemText(hDlg, IDC_UNITNAME, gUnit[HouseCode][UnitCode].Name, 33);
          
          // Exit this dialog box
          EndDialog(hDlg, TRUE);
          break;
        }
      case IDCANCEL :
        {
          EndDialog(hDlg, FALSE);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}


BOOL CALLBACK TimersDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      int lcv;
      
      // Place all the events into the list box
      for (lcv = 0; lcv <= 127; lcv++)
      {
        // If no name is assigned to the current event, then assign its default
        // value of "Event" and the event number
        if (strcmp(gEvt[lcv].Name, "") == 0)
          wsprintf(gEvt[lcv].Name, "Event %i", lcv + 1);
        
        // Add the current event's name to the list box
        SendDlgItemMessage(hDlg, IDC_TIMERS, LB_ADDSTRING, 0, (LPARAM) gEvt[lcv].Name);
      }
      
      // Select the first event
      SendDlgItemMessage(hDlg, IDC_TIMERS, LB_SETCURSEL, 0, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDC_TIMERS :
        {
          // If the user double-clicked, bring up the properties
          if (HIWORD(wParam) != LBN_DBLCLK)
            break;
        }
      case IDC_PROPERTIES :
        {
          // Bring up the unit properties dialog box and refresh if "Ok" was pressed
          int Item = SendDlgItemMessage(hDlg, IDC_TIMERS, LB_GETCURSEL, 0, 0);
          if (DialogBoxParam(ghInstance, "TIMERPROPERTIES", hDlg, (DLGPROC) TimerPropertiesDlgProc, Item))
          {
            if (strcmp(gEvt[Item].Name, "") == 0)
              wsprintf(gEvt[Item].Name, "Event %i", Item + 1);
            SendDlgItemMessage(hDlg, IDC_TIMERS, LB_DELETESTRING, Item, 0);
            SendDlgItemMessage(hDlg, IDC_TIMERS, LB_INSERTSTRING, Item, (LPARAM) gEvt[Item].Name);
            SendDlgItemMessage(hDlg, IDC_TIMERS, LB_SETCURSEL, Item, 0);
            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(IDC_HOUSECODE, LBN_SELCHANGE), 0);
          }
          break;
        }
      case IDCANCEL :
        {
          EndDialog(hDlg, TRUE);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}


BOOL CALLBACK TimerPropertiesDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      int i;
      char String[10];
      
      // Set x equal to the event passed as a parameter
      x = (int) lParam;
      
      // Set the event's name edit control to the name and limit it to 32
      SetDlgItemText(hDlg, IDC_TIMEREVENTNAME, gEvt[x].Name);
      SendDlgItemMessage(hDlg, IDC_TIMEREVENTNAME, EM_LIMITTEXT, 32, 0);
      
      // Set the event enable check box
      CheckDlgButton(hDlg, IDC_EVENTENABLE, gEvt[x].Info.InUse);
      
      // Set the current button states according to the previous values
      switch (gEvt[x].Info.Command)
      {
      case CP290_ON  : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
        CheckDlgButton(hDlg, IDC_ON, TRUE);  break;
      case CP290_OFF : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
        CheckDlgButton(hDlg, IDC_OFF, TRUE);  break;
      case CP290_DIM : EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), TRUE);
        CheckDlgButton(hDlg, IDC_DIM, TRUE);  break;
      }
      
      // Place the percents into the dim level combo-box
      for (i = 15; i >= 0; i--)
      {
        LoadString(ghInstance, i, String, 10);
        SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_ADDSTRING, 0, (LPARAM) String);
      }
      
      // Select the previous dim level
      SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_SETCURSEL, 15 - gEvt[x].Info.DimLevel, 0);
      
      // Set parts of the gCmd global structure to equal the values of the
      // current timer
      gCmd.HouseCode = gEvt[x].Info.HouseCode;
      CopyMemory(gCmd.Unit, gEvt[x].Info.Unit, sizeof (BOOL) * 16);
      CopyMemory(gSelected[gCmd.HouseCode - 'A'], gEvt[x].Info.Unit, sizeof (BOOL) * 16);
      
      // Set the gConfigEvt global structure
      CopyMemory(&gConfigEvt, &gEvt[x].Info, sizeof (CP290TIMEREVENT));
      
      // Copy the hour and minute into their fields
      wsprintf(String, "%i", (int) gEvt[x].Info.Time.wHour);
      SetDlgItemText(hDlg, IDC_HOUR, String);
      wsprintf(String, "%i", (int) gEvt[x].Info.Time.wMinute);
      SetDlgItemText(hDlg, IDC_MINUTE, String);
      
      // Limit the hour and minute fields to 2 characters
      SendDlgItemMessage(hDlg, IDC_HOUR, EM_LIMITTEXT, 2, 0);
      SendDlgItemMessage(hDlg, IDC_MINUTE, EM_LIMITTEXT, 2, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
        // Set the states for the pull-down list box according to the
        // selected command
      case IDC_ON :
      case IDC_OFF :
        {
          EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), FALSE);
          break;
        }
      case IDC_DIM :
        {
          EnableWindow(GetDlgItem(hDlg, IDC_DIMLEVEL), TRUE);
          break;
        }
        
        // Run the ChangeUnits dialog box
      case IDC_CHANGEUNITS :
        {
          DialogBox(ghInstance, "UNITS", hDlg, (DLGPROC) UnitsDlgProc);
          break;
        }
        
        // Run the ChangeDays dialog box
      case IDC_CHANGEDAYS :
        {
          DialogBox(ghInstance, "DAYS", hDlg, (DLGPROC) DaysDlgProc);
          break;
        }
        
        // User pressed "Ok"
      case IDOK :
        {
          BOOL UnitSelected;
          int lcv[2];
          
          // Validate hour
          lcv[0] = GetDlgItemInt(hDlg, IDC_HOUR, NULL, FALSE);
          if (lcv[0] > 23)
          {
            MessageBox(hDlg, "Please enter a valid 24-hour style hour; the hours range from zero to 23.", "Time Error", MB_OK | MB_ICONEXCLAMATION);
            break;
          }
          
          // Validate the minute
          lcv[1] = GetDlgItemInt(hDlg, IDC_MINUTE, NULL, FALSE);
          if (lcv[1] > 59)
          {
            MessageBox(hDlg, "Please enter a valid minute; the minutes range from zero to 59.", "Time Error", MB_OK | MB_ICONEXCLAMATION);
            break;
          }
          
          // Assign the hour and the minute to the structure
          gEvt[x].Info.Time.wHour = (WORD) lcv[0];
          gEvt[x].Info.Time.wMinute = (WORD) lcv[1];
          
          // Get Command Type
          if (IsDlgButtonChecked(hDlg, IDC_ON))
            gEvt[x].Info.Command = CP290_ON;
          if (IsDlgButtonChecked(hDlg, IDC_OFF))
            gEvt[x].Info.Command = CP290_OFF;
          if (IsDlgButtonChecked(hDlg, IDC_DIM))
            gEvt[x].Info.Command = CP290_DIM;
          
          // Get Dim Level
          gEvt[x].Info.DimLevel = 15 - SendDlgItemMessage(hDlg, IDC_DIMLEVEL, CB_GETCURSEL, 0, 0);
          
          // Check to see if any units were selected
          UnitSelected = FALSE;
          for (lcv[0] = 0; lcv[0] <= 15; lcv[0]++)
            for (lcv[1] = 0; lcv[1] <= 15; lcv[1]++)
              if (gSelected[lcv[0]][lcv[1]])
                UnitSelected = TRUE;
              
              // Get the new name for the timer event
              GetDlgItemText(hDlg, IDC_TIMEREVENTNAME, gEvt[x].Name, 33);
              
              // Get the usage flag
              gEvt[x].Info.InUse = IsDlgButtonChecked(hDlg, IDC_EVENTENABLE);
              
              // Copy the house code and the units into the event
              gEvt[x].Info.HouseCode = gCmd.HouseCode;
              CopyMemory(gEvt[x].Info.Unit, gCmd.Unit, sizeof (BOOL) * 16);
              
              // Copy the selected days and mode into gEvt[x]
              CopyMemory(gEvt[x].Info.Day, gConfigEvt.Day, sizeof(BOOL) * 7);
              gEvt[x].Info.Mode = gConfigEvt.Mode;
              
              if (UnitSelected)
              {
                // Interact with the unit and exit if no error messages made their presence known
                if (Interact(hDlg, MODE_TIMEREVENT))
                {
                  // Exit with TRUE
                  EndDialog(hDlg, TRUE);
                }
              }
              else
              {
                // Tell the user to select a unit
                MessageBox(hDlg, "You must select at least one unit to manipulate.", "Unit Error", MB_OK | MB_ICONEXCLAMATION);
              }
              break;
        }
        
        // User pressed "Cancel"
      case IDCANCEL :
        {
          // Exit with FALSE
          EndDialog(hDlg, FALSE);
          break;
        }
        }
        return 0;
     }
  }
  return FALSE;
}


BOOL CALLBACK DaysDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      // Select the days from the current event
      int lcv;
      for (lcv = CP290_SUNDAY; lcv <= CP290_SATURDAY; lcv++)
        if (gConfigEvt.Day[lcv])
          CheckDlgButton(hDlg, lcv + IDC_SUNDAY, TRUE);
        
        // Set the day check marks according to the day method chosen
        switch (gConfigEvt.Mode)
        {
        case CP290_TODAY :
          {
            // Place a check in the today button
            int lcv;
            CheckDlgButton(hDlg, IDC_TODAY, TRUE);
            
            // Disable the ability to select the days
            for (lcv = IDC_SUNDAY; lcv <= IDC_SECURITY; lcv++)
              EnableWindow(GetDlgItem(hDlg, lcv), FALSE);
            break;
          }
        case CP290_TOMORROW :
          {
            // Place a check in the tomorrow button
            int lcv;
            CheckDlgButton(hDlg, IDC_TOMORROW, TRUE);
            
            // Disable the ability to select the days
            for (lcv = IDC_SUNDAY; lcv <= IDC_SECURITY; lcv++)
              EnableWindow(GetDlgItem(hDlg, lcv), FALSE);
            break;
          }
        case CP290_SECURITY :
          {
            // Check-mark the security check box
            CheckDlgButton(hDlg, IDC_SECURITY, TRUE);
          }
        case CP290_CLEAR :
        case CP290_NORMAL :
          {
            // Place a check in the selected days button
            CheckDlgButton(hDlg, IDC_NORMAL, TRUE);
            break;
          }
        }
        return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
        // Set the day check marks according to the day method chosen
      case IDC_TODAY :
      case IDC_TOMORROW :
        {
          // Disable the ability to select the days
          int lcv;
          for (lcv = IDC_SUNDAY; lcv <= IDC_SECURITY; lcv++)
            EnableWindow(GetDlgItem(hDlg, lcv), FALSE);
          break;
        }
      case IDC_NORMAL :
        {
          // Enable the ability to select the days
          int lcv;
          for (lcv = IDC_SUNDAY; lcv <= IDC_SECURITY; lcv++)
            EnableWindow(GetDlgItem(hDlg, lcv), TRUE);
          break;
        }
        
      case IDOK :
        {
          // Get the days
          int lcv;
          for (lcv = CP290_SUNDAY; lcv <= CP290_SATURDAY; lcv++)
            gConfigEvt.Day[lcv] = IsDlgButtonChecked(hDlg, lcv + IDC_SUNDAY);
          
          // Get the mode
          if (IsDlgButtonChecked(hDlg, IDC_NORMAL) && IsDlgButtonChecked(hDlg, IDC_SECURITY))
            gConfigEvt.Mode = CP290_SECURITY;
          if (IsDlgButtonChecked(hDlg, IDC_NORMAL) && !IsDlgButtonChecked(hDlg, IDC_SECURITY))
            gConfigEvt.Mode = CP290_NORMAL;
          if (IsDlgButtonChecked(hDlg, IDC_TODAY))
            gConfigEvt.Mode = CP290_TODAY;
          if (IsDlgButtonChecked(hDlg, IDC_TOMORROW))
            gConfigEvt.Mode = CP290_TOMORROW;
          
          // Exit this dialog box
          EndDialog(hDlg, TRUE);
        }
      case IDCANCEL :
        {
          EndDialog(hDlg, FALSE);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}

void SavePortName(LPSTR szCommPort)
{
  HKEY hKey;
  DWORD Disposition;
  
  RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98",
    0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
    &hKey, &Disposition);
  RegSetValueEx(hKey, "Comm Port", 0, REG_SZ, (LPBYTE) szCommPort, (DWORD) (lstrlen(szCommPort) + 1));
  RegCloseKey(hKey);
}

void LoadPortName(LPSTR szCommPort)
{
  HKEY hKey;
  DWORD Size;
  
  // Set default to com1
  lstrcpy(szCommPort, "COM1");
  
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98",
    0, KEY_READ, &hKey) == ERROR_SUCCESS)
  {
    DWORD Type;
    
    Size = (DWORD) (7);
    RegQueryValueEx(hKey, "Comm Port", NULL, &Type, (LPBYTE) szCommPort, &Size);
    RegCloseKey(hKey);
  }
}

BOOL Interact(HWND hWnd, DWORD Action)
{
  // Bring up the "Interacting With CP290" message
  DWORD ReturnValue;
  DialogBoxParam(ghInstance, "BUSY", hWnd, (DLGPROC) InteractingDlgProc, (LPARAM) Action);
  
  // Wait for the interaction thread to exit (up to 1 sec) and then close it
  if (WaitForSingleObject(ghThread, 1000) == WAIT_TIMEOUT)
    // Terminate the thread (thread is probably locked); terminate with an error
    TerminateThread(ghThread, 0);
 
  // Get the thread's success story and return it to the calling procedure
  GetExitCodeThread(ghThread, &ReturnValue);
  CloseHandle(ghThread);
  return ReturnValue;
}

DWORD InteractAnimationThread(DWORD unused) {
  // Perform an animation while ghInteractingDlg is valid
  HICON icons[4];
  DWORD frame;

  // Load the icons into memory
  for (frame = 0; frame <= 3; frame++)
	icons[frame] = LoadIcon(ghInstance, MAKEINTRESOURCE(frame + IDI_INTERACTANIMATION1));

  // Animation loop
  frame = 0;
  while (ghInteractingDlg) {
	if (frame > 3)
	  frame = 0;

	// Set the icon
	SendDlgItemMessage(ghInteractingDlg, IDC_INTERACTANIMATION, STM_SETICON, (WPARAM) icons[frame], 0);

	// Sleep for .75 seconds and then advance frame
	Sleep(750);
	frame++;
  }

  // Set the AnimationThreadID to NULL
  AnimationThreadID = 0;
  return 0;
}


BOOL CALLBACK InteractingDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
	  // Run the animation, after closing an existing one
      if (AnimationThreadID) {
	    // Wait for it to exit (up to 1 sec) and then close it
	    ghInteractingDlg = NULL;
	    if (WaitForSingleObject(ghAnimationThread, 1000) == WAIT_TIMEOUT)
	      TerminateThread(ghAnimationThread, 0);
	    CloseHandle(ghAnimationThread);
	  }
      ghInteractingDlg = hDlg;
      ghAnimationThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) InteractAnimationThread, 0, 0, &AnimationThreadID);

	  // Load the thread, if we're allowed to
      ghThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) InteractingThread, (LPVOID) lParam, 0, &ThreadID);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDCANCEL :
        {
          // Initiate a cancelation
		  ThreadID = 0;
		  SetDlgItemText(hDlg, IDC_ACTION, "Canceling operation (wait)");
		  EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
          break;
        }
      case 300 :
        {
          // Thread exited (or another program explicity called this), exit
          // dialog box as a result
		  ghInteractingDlg = NULL;
          EndDialog(hDlg, 0);
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}

void SaveGlobals(void)
{
  // Save the timer events and the unit names into the registry
  HKEY hParentKey;
  HKEY hHouseKey;
  HKEY hUnitKey;
  HKEY hEventKey;
  char String[20];
  DWORD Disposition;
  int lcv[2];
  
  // Open the parent key for the units
  RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98\\Units",
    0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
    &hParentKey, &Disposition);
  
  // Save all the units
  for (lcv[0] = 0; lcv[0] <= 15; lcv[0]++)
  {
    // Open the key for each house code
    wsprintf(String, "House %c", (char) (lcv[0] + 'A'));
    RegCreateKeyEx(hParentKey, String, 0, NULL, REG_OPTION_NON_VOLATILE,
      KEY_ALL_ACCESS, NULL, &hHouseKey, &Disposition);
    
    for (lcv[1] = 0; lcv[1] <= 15; lcv[1]++)
    {
      // Open the key for each unit and save the name
      wsprintf(String, "Unit %i", lcv[1] + 1);
      RegCreateKeyEx(hHouseKey, String, 0, NULL, REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS, NULL, &hUnitKey, &Disposition);
      RegSetValueEx(hUnitKey, "Name", 0, REG_SZ, (LPBYTE) gUnit[lcv[0]][lcv[1]].Name, (DWORD) (lstrlen(gUnit[lcv[0]][lcv[1]].Name) + 1));
      
      // Close the unit key
      RegCloseKey(hUnitKey);
    }
    
    // Close the house code key
    RegCloseKey(hHouseKey);
  }
  
  // Close the parent key
  RegCloseKey(hParentKey);
  
  // Open the parent key for the timer events
  RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98\\Timer Events",
    0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
    &hParentKey, &Disposition);
  
  for (lcv[0] = 0; lcv[0] <= 127; lcv[0]++)
  {
    // Open the key for each timer event
    wsprintf(String, "Timer Event %i", lcv[0] + 1);
    RegCreateKeyEx(hParentKey, String, 0, NULL, REG_OPTION_NON_VOLATILE,
      KEY_ALL_ACCESS, NULL, &hEventKey, &Disposition);
    
    // Save the name
    RegSetValueEx(hEventKey, "Name", 0, REG_SZ, (LPVOID) gEvt[lcv[0]].Name, (DWORD) (lstrlen(gEvt[lcv[0]].Name) + 1));
    
    // Save the data structure
    RegSetValueEx(hEventKey, "Data", 0, REG_BINARY, (LPVOID) &gEvt[lcv[0]].Info, (DWORD) sizeof(CP290TIMEREVENT));
    
    // Close the timer event key
    RegCloseKey(hEventKey);
  }
  
  // Close the parent key
  RegCloseKey(hParentKey);
}

void LoadGlobals(void)
{
  // Load the timer events and the unit names from the registry
  CP290TIMEREVENT cpEvent;
  HKEY hParentKey;
  HKEY hHouseKey;
  HKEY hUnitKey;
  HKEY hEventKey;
  char String[20];
  DWORD Size;
  DWORD Type;
  BOOL UserApproval = FALSE;
  int lcv[2];
  
  // Open the parent key for the units
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98\\Units",
    0, KEY_READ, &hParentKey) == ERROR_SUCCESS)
  {
    // Load all the units
    for (lcv[0] = 0; lcv[0] <= 15; lcv[0]++)
    {
      // Open the key for each house code
      wsprintf(String, "House %c", (char) (lcv[0] + 'A'));
      if (RegOpenKeyEx(hParentKey, String, 0, KEY_READ, &hHouseKey) == ERROR_SUCCESS)
      {
        for (lcv[1] = 0; lcv[1] <= 15; lcv[1]++)
        {
          // Open the key for each unit and get the name
          wsprintf(String, "Unit %i", lcv[1] + 1);
          if (RegOpenKeyEx(hHouseKey, String, 0, KEY_READ, &hUnitKey) == ERROR_SUCCESS)
          {
            Size = 33;
            RegQueryValueEx(hUnitKey, "Name", NULL, &Type, (LPBYTE) gUnit[lcv[0]][lcv[1]].Name, &Size);
            
            // Close the unit key
            RegCloseKey(hUnitKey);
          }
        }
        
        // Close the house code key
        RegCloseKey(hHouseKey);
      }
    }
    
    // Close the parent key
    RegCloseKey(hParentKey);
  }
  
  // Open the parent key for the timer events
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 98\\Timer Events",
    0, KEY_READ, &hParentKey) == ERROR_SUCCESS)
  {
    for (lcv[0] = 0; lcv[0] <= 127; lcv[0]++)
    {
      // Open the key for each timer event
      wsprintf(String, "Timer Event %i", lcv[0] + 1);
      if (RegOpenKeyEx(hParentKey, String, 0, KEY_READ, &hEventKey) == ERROR_SUCCESS)
      {
        // Load the name and the data structure
        Size = 33;
        RegQueryValueEx(hEventKey, "Name", NULL, &Type, (LPBYTE) gEvt[lcv[0]].Name, &Size);
        Size = sizeof(CP290TIMEREVENT);
        if (RegQueryValueEx(hEventKey, "Data", NULL, &Type, (LPBYTE) &cpEvent, &Size) == ERROR_SUCCESS)
        {
          // If the unit's event is not in use, then we'll consider some special conditions...
          if (!gEvt[lcv[0]].Info.InUse)
          {
            // If the mode was set to TODAY or TOMORROW (temp modes), then we'll
            // set the usage flag to FALSE
            if (((cpEvent.Mode == CP290_TODAY) || (cpEvent.Mode == CP290_TOMORROW)) && cpEvent.InUse)
              cpEvent.InUse = FALSE;
            
            // If the stored event is not in use (or if it had TODAY or TOMORROW set,
            // but the event had already expired from the unit), then copy the
            // stored event into the timers
            if (!cpEvent.InUse)
              CopyMemory(&gEvt[lcv[0]].Info, &cpEvent, sizeof(CP290TIMEREVENT));
            
            // If the saved value is in use and the user has not been asked whether or not
            // to use them, then let's do that...
            if ((cpEvent.InUse) && (UserApproval == FALSE) && !ErrorFlag)
            {
              // Ask the user if the registry's values should be used
              if (MessageBox(HWND_DESKTOP, "The unit is missing one or more events that the computer has.\n\nDo you want to update the unit's memory with the missing events?", "Missing Events", MB_YESNO | MB_ICONQUESTION) == IDYES)
                UserApproval = TRUE;
              else
                UserApproval = 2;
            }
            
            // If we have user approval (and the timer is in use)...
            if ((cpEvent.InUse) && (UserApproval == TRUE) && !ErrorFlag)
            {
              // Copy the registry's value into the unit and into the array
              CopyMemory(&gEvt[lcv[0]].Info, &cpEvent, sizeof(CP290TIMEREVENT));
              x = lcv[0];
              if (!Interact(HWND_DESKTOP, MODE_TIMEREVENT))
				ErrorFlag = TRUE;
            }

			// If there is an error condition, then just use the registry for now
			if (ErrorFlag)
              CopyMemory(&gEvt[lcv[0]].Info, &cpEvent, sizeof(CP290TIMEREVENT));
          }
        }
        
        // Close the timer event key
        RegCloseKey(hEventKey);
      }
    }
    
    // Close the parent key
    RegCloseKey(hParentKey);
  }
}

BOOL CALLBACK ResetAskDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
  case WM_INITDIALOG :
    {
      CheckDlgButton(hDlg, IDC_RETAINMEMORY, TRUE);
      CheckDlgButton(hDlg, IDC_RETAINEVENTS, TRUE);
      CheckDlgButton(hDlg, IDC_RETAINICONS, TRUE);
      SetDlgItemText(hDlg, IDC_PORT, (LPCSTR) lParam);
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "No Change");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "A");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "B");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "C");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "D");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "E");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "F");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "G");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "H");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "I");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "J");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "K");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "L");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "M");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "N");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "O");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "P");
      SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_SETCURSEL, 0, 0);
      return TRUE;
    }
  case WM_COMMAND :
    {
      switch (LOWORD(wParam))
      {
      case IDC_RETAINMEMORY :
        {
          if (IsDlgButtonChecked(hDlg, IDC_RETAINMEMORY))
          {
            EnableWindow(GetDlgItem(hDlg, IDC_RETAINICONS), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_RETAINEVENTS), TRUE);
            CheckDlgButton(hDlg, IDC_RETAINEVENTS, TRUE);
            CheckDlgButton(hDlg, IDC_RETAINICONS, TRUE);
          }
          else
          {
            EnableWindow(GetDlgItem(hDlg, IDC_RETAINICONS), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_RETAINEVENTS), FALSE);
            CheckDlgButton(hDlg, IDC_RETAINEVENTS, FALSE);
            CheckDlgButton(hDlg, IDC_RETAINICONS, FALSE);
          }
          break;
        }
      case IDCANCEL :
        {
          EndDialog(hDlg, FALSE);
          break;
        }
      case IDC_RESET :
        {
          RetainMemory = IsDlgButtonChecked(hDlg, IDC_RETAINMEMORY);
          RetainIcons = IsDlgButtonChecked(hDlg, IDC_RETAINICONS);
          RetainEvents = IsDlgButtonChecked(hDlg, IDC_RETAINEVENTS);
          NewHouseCode = (char) (SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0) + 'A' - 1);
          if (RetainMemory)
          {
            if (MessageBox(hDlg, "In the unlikely event of a unit failure, your data could be erased.  Be sure that you have backed up your data before running this program.  Are you sure you want to reset?", "Reset Warning", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
              EndDialog(hDlg, TRUE);
          }
          else
          {
            if (MessageBox(hDlg, "Resetting without retaining the memory means that your unit will no longer contain any events and icons.  Are you sure you want to reset?", "Reset Warning", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
              EndDialog(hDlg, TRUE);
          }
          break;
        }
      }
      return 0;
    }
  }
  return FALSE;
}
