// Reset

// Purpose: This program reads the unit's memory into the computer's memory,
//          resets the unit, and writes the data back into the unit.

#include <windows.h>
#include "x10dll.h"
#include "reset.h"

BOOL CALLBACK ResetAskDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ResettingDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
void DisplayError(HWND hDlg, LPCSTR Title);
int ResetUnit(HWND hDlg);
void SavePortName(LPSTR szCommPort);
void LoadPortName(LPSTR szCommPort);

HANDLE hThread;
BOOL RetainMemory;
BOOL RetainIcons;
BOOL RetainEvents;
char gPort[5];
char NewHouseCode;


int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
  MSG msg;

  // Ask the user for reset confirmation
  if (DialogBox(hInstance, "RESETASK", HWND_DESKTOP, (DLGPROC) ResetAskDlgProc))
  {
	 DWORD ThreadID;
	 HWND hDlg;

	 hDlg = CreateDialog(hInstance, "RESETTING", HWND_DESKTOP, (DLGPROC) ResettingDlgProc);
    SetActiveWindow(hDlg);
	 hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) ResetUnit, hDlg, 0, &ThreadID);

	 while (GetMessage(&msg, NULL, 0, 0))
	 {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	 }

	 CloseHandle(hThread);
  }
  return msg.wParam;
}

void DisplayError(HWND hDlg, LPCSTR Title)
{
  char ErrorMessage[CP290ERR_MESSAGELENGTH];
  CP290_GetLastError(ErrorMessage);
  MessageBeep(MB_ICONSTOP);
  MessageBox(hDlg, ErrorMessage, Title, MB_OK | MB_ICONSTOP);
  SetDlgItemText(hDlg, IDC_ACTION, "Exiting");
  SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
  ExitThread(0);
}

int ResetUnit(HWND hDlg)
{
  CP290TIMEREVENT cpEvent[128];
  CP290ICON cpIcon[256];
  SYSTEMTIME cpTime;
  BOOL ErrorFlag;
  char HouseCode;
  int lcv;

  SetThreadPriority(hThread, THREAD_PRIORITY_HIGHEST);

  SetDlgItemText(hDlg, IDC_ACTION, "Initializing");
  if (!CP290_Initialize(gPort))
	 DisplayError(hDlg, "Problem During Initiailization");

  if (RetainMemory && RetainEvents)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Reading Events");
	 if (!CP290_GetTimerEvents(cpEvent, 128))
		DisplayError(hDlg, "Problem During Event Read");
  }
  if (RetainMemory && RetainIcons)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Reading Icons");
	 if (!CP290_GetIcons(cpIcon, 256))
		DisplayError(hDlg, "Problem During Icon Read");
  }
  if (RetainMemory)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Reading Time");
	 if (!CP290_GetTime(&cpTime))
		DisplayError(hDlg, "Problem During Time Read");
	 SetDlgItemText(hDlg, IDC_ACTION, "Reading House Code");
	 if (!(HouseCode = CP290_GetBaseHouseCode()))
		DisplayError(hDlg, "Problem During House Code Read");
  }

  SetDlgItemText(hDlg, IDC_ACTION, "Resetting");
  if (!CP290_Reset())
  {
	 MessageBeep(MB_ICONSTOP);
	 if (CP290_GetLastError(NULL) == CP290ERR_WRITE)
	 {
		MessageBox(hDlg, "There was a problem during the reset.  However, your data is completely safe.  Please check the connection to the unit and try again.", "Problem During Reset", MB_OK | MB_ICONSTOP);
		SetDlgItemText(hDlg, IDC_ACTION, "Exiting");
		SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
		ExitThread(0);
	 }
	 else
	 {
		MessageBox(hDlg, "There was an undocumented problem during the reset, which means that you are using a newer version of X10DLL.DLL.  Your data might have been erased.  Please check the connection to the unit and try again.  If you chose to retain the settings, then I will restore them.", "Problem During Reset", MB_OK | MB_ICONSTOP);
		if (!RetainMemory)
		{
		  SetDlgItemText(hDlg, IDC_ACTION, "Exiting");
		  SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
		  ExitThread(0);
		}
	 }
  }

  if (RetainMemory)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Writing House Code");
	 if (NewHouseCode != 'A' - 1)
		HouseCode = NewHouseCode;
	 do
	 {
		ErrorFlag = !CP290_SetBaseHouseCode(HouseCode);
		if (ErrorFlag)
		{
		  char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
		  MessageBeep(MB_ICONEXCLAMATION);
		  CP290_GetLastError(ErrorMessage);
		  lstrcat(ErrorMessage, ".  The house code might not have been set properly.  Do you want to try again?");
		  if (MessageBox(hDlg, ErrorMessage, "Problem During House Code Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			 ErrorFlag = FALSE;
		}
	 } while (ErrorFlag);

	 SetDlgItemText(hDlg, IDC_ACTION, "Writing Time");
	 do
	 {
		ErrorFlag = !CP290_SetTime(&cpTime);
		if (ErrorFlag)
		{
		  char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
		  MessageBeep(MB_ICONEXCLAMATION);
		  CP290_GetLastError(ErrorMessage);
		  lstrcat(ErrorMessage, ".  The time might not have been set properly.  Do you want to try again?");
		  if (MessageBox(hDlg, ErrorMessage, "Problem During Time Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			 ErrorFlag = FALSE;
		}
	 } while (ErrorFlag);
  }

  if (RetainMemory && RetainEvents)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Writing Events");
	 do
	 {
		ErrorFlag = FALSE;
		for (lcv = 0; lcv <= 127; lcv++)
		  if (cpEvent[lcv].InUse)
			 if (!CP290_SetTimerEvent(&cpEvent[lcv]))
				ErrorFlag = TRUE;
		if (ErrorFlag)
		{
		  char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
		  MessageBeep(MB_ICONEXCLAMATION);
		  CP290_GetLastError(ErrorMessage);
		  lstrcat(ErrorMessage, ".  One or more of your events could not be written properly.  Do you want to try again?");
		  if (MessageBox(hDlg, ErrorMessage, "Problem During Event Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			 ErrorFlag = FALSE;
		}
	 } while (ErrorFlag);
  }

  if (RetainMemory && RetainIcons)
  {
	 SetDlgItemText(hDlg, IDC_ACTION, "Writing Icons");
	 do
	 {
		ErrorFlag = FALSE;
		for (lcv = 0; lcv <= 255; lcv++)
		  if (cpIcon[lcv].InUse)
			 if (!CP290_SetIcon(&cpIcon[lcv]))
				ErrorFlag = TRUE;
		if (ErrorFlag)
		{
		  char ErrorMessage[CP290ERR_MESSAGELENGTH + 150];
		  MessageBeep(MB_ICONEXCLAMATION);
		  CP290_GetLastError(ErrorMessage);
		  lstrcat(ErrorMessage, ".  One or more of your icons could not be written properly.  Do you want to try again?");
		  if (MessageBox(hDlg, ErrorMessage, "Problem During Icon Write", MB_YESNO | MB_ICONEXCLAMATION) == IDNO)
			 ErrorFlag = FALSE;
		}
	 } while (ErrorFlag);
  }

  if (!RetainMemory)
  {
	 MessageBeep(MB_ICONQUESTION);
	 if (MessageBox(hDlg, "Would you like to initialize the unit's values to their defaults?", "Initialization", MB_YESNO | MB_ICONQUESTION) == IDYES)
	 {
		SetDlgItemText(hDlg, IDC_ACTION, "Writing House Code");
		if (NewHouseCode != 'A' - 1)
		  CP290_SetBaseHouseCode(NewHouseCode);
		else
		  CP290_SetBaseHouseCode('A');
		SetDlgItemText(hDlg, IDC_ACTION, "Writing Time");
		GetLocalTime(&cpTime);
		CP290_SetTime(&cpTime);
	 }
  }

  SetDlgItemText(hDlg, IDC_ACTION, "Uninitializing");
  CP290_Uninitialize();

  SetDlgItemText(hDlg, IDC_ACTION, "Exiting");
  SendMessage(hDlg, WM_COMMAND, MAKEWPARAM(300, NULL), (LPARAM) NULL);
  ExitThread(0);
  return 0;
}


BOOL CALLBACK ResetAskDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_INITDIALOG :
	 {
		CheckDlgButton(hDlg, IDC_RETAINMEMORY, TRUE);
		CheckDlgButton(hDlg, IDC_RETAINEVENTS, TRUE);
		CheckDlgButton(hDlg, IDC_RETAINICONS, TRUE);
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM1");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM2");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM3");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM4");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM5");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM6");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM7");
		SendDlgItemMessage(hDlg, IDC_PORT, CB_ADDSTRING, 0, (LPARAM) "COM8");
		LoadPortName(gPort);
		SendDlgItemMessage(hDlg, IDC_PORT, CB_SELECTSTRING, 0, (LPARAM) gPort);
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "No Change");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "A");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "B");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "C");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "D");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "E");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "F");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "G");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "H");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "I");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "J");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "K");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "L");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "M");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "N");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "O");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_ADDSTRING, 0, (LPARAM) "P");
		SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_SETCURSEL, 0, 0);
		return TRUE;
	 }
	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case IDC_RETAINMEMORY :
		  {
			 if (IsDlgButtonChecked(hDlg, IDC_RETAINMEMORY))
			 {
				EnableWindow(GetDlgItem(hDlg, IDC_RETAINICONS), TRUE);
				EnableWindow(GetDlgItem(hDlg, IDC_RETAINEVENTS), TRUE);
				CheckDlgButton(hDlg, IDC_RETAINEVENTS, TRUE);
				CheckDlgButton(hDlg, IDC_RETAINICONS, TRUE);
			 }
			 else
			 {
				EnableWindow(GetDlgItem(hDlg, IDC_RETAINICONS), FALSE);
				EnableWindow(GetDlgItem(hDlg, IDC_RETAINEVENTS), FALSE);
				CheckDlgButton(hDlg, IDC_RETAINEVENTS, FALSE);
				CheckDlgButton(hDlg, IDC_RETAINICONS, FALSE);
			 }
			 break;
		  }
		  case IDCANCEL :
		  {
			 EndDialog(hDlg, FALSE);
			 break;
		  }
		  case IDC_RESET :
		  {
			 RetainMemory = IsDlgButtonChecked(hDlg, IDC_RETAINMEMORY);
			 RetainIcons = IsDlgButtonChecked(hDlg, IDC_RETAINICONS);
			 RetainEvents = IsDlgButtonChecked(hDlg, IDC_RETAINEVENTS);
			 SendDlgItemMessage(hDlg, IDC_PORT, CB_GETLBTEXT, (WPARAM) SendDlgItemMessage(hDlg, IDC_PORT, CB_GETCURSEL, 0, 0), (LPARAM) gPort);
			 SavePortName(gPort);
			 NewHouseCode = (char) (SendDlgItemMessage(hDlg, IDC_HOUSECODE, CB_GETCURSEL, 0, 0) + 'A' - 1);
			 if (RetainMemory)
			 {
				MessageBeep(MB_ICONEXCLAMATION);
				if (MessageBox(hDlg, "In the unlikely event of a unit failure, your data could be erased.  Be sure that you have backed up your data before running this program.  Are you sure you want to reset?", "Reset Warning", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
				  EndDialog(hDlg, TRUE);
			 }
			 else
			 {
				MessageBeep(MB_ICONEXCLAMATION);
				if (MessageBox(hDlg, "Resetting without retaining the memory means that your unit will no longer contain any events and icons.  Are you sure you want to reset?", "Reset Warning", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
				  EndDialog(hDlg, TRUE);
			 }
			 break;
		  }
		}
		return 0;
	 }
  }
  return FALSE;
}


BOOL CALLBACK ResettingDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	 case WM_INITDIALOG :
	 {
		return TRUE;
	 }
	 case WM_COMMAND :
	 {
		switch (LOWORD(wParam))
		{
		  case IDCANCEL :
		  {
			 MessageBeep(MB_ICONEXCLAMATION);
			 if (MessageBox(hDlg, "Are you sure you want to cancel the reset?  Doing so will result in a loss of data (YOU HAVE BEEN WARNED!)", "Cancellation Danger", MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
			 {
				TerminateThread(hThread, 0);
				DestroyWindow(hDlg);
				PostQuitMessage(0);
			 }
          break;
		  }
		  case 300 :
		  {
			 DestroyWindow(hDlg);
			 PostQuitMessage(0);
			 break;
		  }
		}
		return 0;
	 }
  }
  return FALSE;
}

void SavePortName(LPSTR szCommPort)
{
  HKEY hKey;
  DWORD Disposition;

  RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 96",
					  0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
					  &hKey, &Disposition);
  RegSetValueEx(hKey, "Comm Port", 0, REG_SZ, (LPBYTE) szCommPort, (DWORD) (lstrlen(szCommPort) + 1));
  RegCloseKey(hKey);
}

void LoadPortName(LPSTR szCommPort)
{
  HKEY hKey;
  DWORD Size;

  // Set default to com1
  lstrcpy(szCommPort, "COM1");

  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Steven Lawrance\\Home Control 96",
					0, KEY_READ, &hKey) == ERROR_SUCCESS)
  {
	 DWORD Type;

	 Size = (DWORD) (4);
	 RegQueryValueEx(hKey, "Comm Port", NULL, &Type, (LPBYTE) szCommPort, &Size);
	 RegCloseKey(hKey);
  }
}

