// X10 CP290 Dynamic Link Library
// Header File

// Constant Declarations
#define CP290_ON        0
#define CP290_OFF       1
#define CP290_DIM       2
#define CP290_NORMAL    0
#define CP290_SECURITY  1
#define CP290_TODAY     2
#define CP290_TOMORROW  3
#define CP290_CLEAR     4
#define CP290_SUNDAY    0
#define CP290_MONDAY    1
#define CP290_TUESDAY   2
#define CP290_WEDNESDAY 3
#define CP290_THURSDAY  4
#define CP290_FRIDAY    5
#define CP290_SATURDAY  6

// Error Message Constants
#define CP290ERR_WRITE          1
#define CP290ERR_READ           2
#define CP290ERR_CHECKSUM       3
#define CP290ERR_NO_ACK         4
#define CP290ERR_PORT_OPEN      5
#define CP290ERR_COMM_DCB       6
#define CP290ERR_COMM_STATE     7
#define CP290ERR_BAD_DAY        8
#define CP290ERR_BAD_HOUR       9
#define CP290ERR_BAD_MINUTE     10
#define CP290ERR_BAD_ID         11
#define CP290ERR_BAD_HOUSECODE  12
#define CP290ERR_BAD_COMMAND    13
#define CP290ERR_BAD_MODE       14
#define CP290ERR_BAD_DIMLEVEL   15
#define CP290ERR_BAD_DATABYTE   16
#define CP290ERR_MESSAGELENGTH  50


// Type Declarations
typedef struct _CP290ICON
{
  BYTE IconId;
  BYTE Data[2];
  BOOL InUse;
} CP290ICON, *LPCP290ICON;
typedef struct _CP290COMMAND
{
  int DimLevel;
  int Command;
  char HouseCode;
  BOOL Unit[16];
} CP290COMMAND, *LPCP290COMMAND;
typedef struct _CP290TIMEREVENT
{
  BYTE EventId;
  int Mode;
  SYSTEMTIME Time;
  BOOL Unit[16];
  BOOL Day[7];
  char HouseCode;
  int DimLevel;
  int Command;
  BOOL InUse;
} CP290TIMEREVENT, *LPCP290TIMEREVENT;


// Function Prototypes
#ifdef __cplusplus
extern "C" {
#endif
char CP290_GetBaseHouseCode(void);
BOOL CP290_SetBaseHouseCode(char BaseHouseCode);
BOOL CP290_GetTimerEvents(CP290TIMEREVENT *TimerEvent, int NumEvents);
BOOL CP290_SetTimerEvent(CP290TIMEREVENT *TimerEvent);
BOOL CP290_GetTime(SYSTEMTIME *Time);
BOOL CP290_SetTime(SYSTEMTIME *Time);
BOOL CP290_GetIcons(CP290ICON *Icon, int NumIcons);
BOOL CP290_SetIcon(CP290ICON *Icon);
BOOL CP290_Control(CP290COMMAND *CmdStruct);
BOOL CP290_Initialize(LPCSTR DeviceName);
BOOL CP290_Uninitialize(void);
DWORD CP290_GetLastError(LPSTR MessageText);
BOOL CP290_Reset(void);
#ifdef __cplusplus
}
#endif
